const { app, BrowserWindow, Menu, dialog, Notification } = require('electron');
const fs                                                 = require('fs');
var window                                               = null
var util                                                 = require('util');
var pathutil                                             = require('path');

var fxmanifest = [
    "fx_version 'adamant'",
    "game 'gta5'",
    "lua54 'on'",
    " ",
    "description 'Generated by Scriptorium Vehicles Organiser'",
    "author 'The Scriptorium'",
    " ",
    "files {",
    "    'data/**/*.meta'",
    "}",
    "data_file 'HANDLING_FILE' 'data/**/handling.meta'",
    "data_file 'VEHICLE_LAYOUTS_FILE' 'data/**/vehiclelayouts.meta'",
    "data_file 'VEHICLE_METADATA_FILE' 'data/**/vehicles.meta'",
    "data_file 'CARCOLS_FILE' 'data/**/carcols.meta'",
    "data_file 'VEHICLE_VARIATION_FILE' 'data/**/carvariations.meta'",
    "data_file 'DLCTEXT_FILE' 'data/**/dlctext.meta'",
    "data_file 'CARCONTENTUNLOCKS_FILE' 'data/**/carcontentunlocks.meta'"
]
const manifestString = () =>
{
    var str = '';
    for (var x of fxmanifest)
    {
        str += x + "\n";
    }
    return str;
}
const __init = () =>
{
    window = new BrowserWindow({
        width: 600,
        height: 600,
        webPreferences: {
            nodeIntegration: true,
            contextIsolation: false
        },
        resizable: false,
        icon: 'src/assets/logo.png'
    });
    window.setTitle('Scriptorium | Vehicles Organiser');
    win = window;
    win.loadFile('src/ui.html');
    __loadMenu(win);
};
const copyFiles = (src = String, dst = String, files = Array) =>
{
    return Promise.all(files.map((f) =>
    {
        return util.promisify(fs.copyFile)(pathutil.join(src, f), pathutil.join(dst, f));
    }));
}
const __loadMenu = (win) =>
{
    var items = [
        {
            label: 'Vehicles',
            submenu: [
                {
                    label: 'Add vehicle',
                    click: async () =>
                    {
                        __addVehicle(win);
                    }
                }
            ]
        },
        {
            label: 'Export',
            submenu: [
                {
                    label: 'FiveM',
                    click: async () =>
                    {
                        if (vehicles.length < 1) return;
                        dialog.showOpenDialog(win, {
                            properties: ['openDirectory']
                        }).then((data) =>
                        {
                            if (!data.canceled)
                            {
                                var url    = data.filePaths[0];
                                var exists = fs.existsSync(url + '/vehicles_export');
                                if (exists)
                                {
                                    new Notification({
                                        title: 'Error!',
                                        body: 'Remove old export folder.',
                                        icon: 'src/assets/logo.png'
                                    }).show();
                                    return;
                                }
                                var start = new Date();
                                fs.mkdirSync(url + '/vehicles_export', {});
                                fs.mkdirSync(url + '/vehicles_export/stream', {});
                                fs.mkdirSync(url + '/vehicles_export/data', {});
                                for (var veh of vehicles)
                                {
                                    fs.mkdirSync(url + "/vehicles_export/stream/" + veh.model);
                                    fs.mkdirSync(url + "/vehicles_export/stream/" + veh.model + "/mods");
                                    fs.mkdirSync(url + "/vehicles_export/data/" + veh.model);
                                    if (veh.meta.length > 0)
                                    {
                                        var files = [];
                                        for (var file of veh.meta)
                                        {
                                            files.push(file[1]);
                                        }
                                        var input  = file[0].replace("/" + file[1], '');
                                        var output = url + "/vehicles_export/data/" + veh.model; 
                                        copyFiles(input, output, files);
                                    }
                                    if (veh.stream.length > 0)
                                    {
                                        var files = [];
                                        for (var file of veh.stream)
                                        {
                                            files.push(file[1]);
                                        }
                                        var input  = file[0].replace("/" + file[1], '');
                                        var output = url + "/vehicles_export/stream/" + veh.model; 
                                        copyFiles(input, output, files);
                                    }
                                    if (veh.mods.length > 0)
                                    {
                                        var files = [];
                                        for (var file of veh.mods)
                                        {
                                            files.push(file[1]);
                                        }
                                        var input  = file[0].replace("/" + file[1], '');
                                        var output = url + "/vehicles_export/stream/" + veh.model + "/mods"; 
                                        copyFiles(input, output, files);
                                    }
                                }
                                fs.writeFileSync(url + '/vehicles_export/fxmanifest.lua', manifestString(), {});
                                var time = new Date() - start;
                                new Notification({
                                    title: 'Script exported!',
                                    body: 'It took ' + time + "ms.",
                                    icon: 'src/assets/logo.png'
                                }).show();
                            }
                        });
                    }
                }
            ]
        },
        {
            label: 'Project',
            submenu: [
                {
                    label: 'Save',
                    click: async () =>
                    {
                        if (vehicles.length < 1) return;
                        var path = dialog.showSaveDialogSync(window, {
                            defaultPath: 'C:/new-project.scvehicles',
                            filters: [
                                {name: 'Scriptorium Vehicles Project', extensions: ['scvehicles']}
                            ]
                        });
                        if (path !== undefined)
                        {
                            var vehdata = JSON.stringify(vehicles);
                            fs.writeFileSync(path, vehdata, {});
                            new Notification({
                                title: 'Project saved!',
                                body: path,
                                icon: 'src/assets/logo.png'
                            }).show();
                        }
                    }
                },
                {
                    label: 'Load',
                    click: async () =>
                    {
                        dialog.showOpenDialog(win, {
                            properties: ['openFile'],
                            filters: [
                                {name: 'Scriptorium Vehicles Project', extensions: ['scvehicles']}
                            ]
                        }).then((data) =>
                        {
                            if (!data.canceled)
                            {
                                var url     = data.filePaths[0];
                                var vehdata = JSON.parse(fs.readFileSync(url, 'utf-8'));
                                vehicles    = vehdata;
                                __updateList(); 
                            }
                        });
                    }
                }
            ]
        },
    ]
    var menu = Menu.buildFromTemplate(items);
    Menu.setApplicationMenu(menu);
};
var vehicles = [];
const __addVehicle = (win) =>
{ 
    dialog.showOpenDialog(win, {
        properties: ['openDirectory']
    }).then((data) =>
    {
        if (!data.canceled)
        {
            var folder  = data.filePaths[0];
            var files   = fs.readdirSync(folder);
            var vehicle = {
                model: '',
                meta: [],
                stream: [],
                mods: []
            };
            for (var file of files)
            {
                var parts = file.split('.');
                if (vehicle.model === '' && parts[1] === 'ytd')
                {
                    vehicle.model = parts[0].split('+')[0];
                }
                var url = folder + "/" + file;
                if (parts[1] === 'ytd' || parts[1] === 'yft' || parts[1] === 'ydd' || parts[1] === 'ydr')
                {
                    vehicle.stream[vehicle.stream.length] = [url, file];
                }
                else if (parts[1] === 'meta')
                {
                    vehicle.meta[vehicle.meta.length] = [url, file];
                }
            }
            if (fs.existsSync(folder + "/mods"))
            {
                var mods = fs.readdirSync(folder + "/mods");
                for (var mod of mods)
                {
                    var url = folder + "/mods/" + mod;
                    vehicle.mods[vehicle.mods.length] = [url, mod];
                }
            }
            vehicles.push(vehicle);
            __updateList();
        }
    });
};
app.whenReady().then(() =>
{
    if (process.platform === 'win32')
    {
        app.setAppUserModelId('Scriptorium | Vehicles Organiser');
    }
    __init();
});
app.on('window-all-closed', () =>
{
    app.quit();
});

/* CALLBACKS */
const ipc = require('electron').ipcMain;
ipc.on('removeVehicle', (e, model) =>
{
    var index = -1;
    for (var i = 0; i < vehicles.length; i++)
    {
        var row = vehicles[i];
        if (row.model === model)
        {
            index = i;
            break;
        }
    }
    vehicles.splice(index, 1);
    __updateList();
});
const __updateList = () =>
{
    window.webContents.send('updateList', vehicles);
};